@@ -4,6 +4,7 @@
 /* make the include files below expose their flags */
 #define HA_EXPOSE_FLAGS
 
+#include <haproxy/applet-t.h>
 #include <haproxy/channel-t.h>
 #include <haproxy/connection-t.h>
 #include <haproxy/fd-t.h>
@@ -12,6 +13,8 @@
 #include <haproxy/mux_fcgi-t.h>
 #include <haproxy/mux_h2-t.h>
 #include <haproxy/mux_h1-t.h>
+#include <haproxy/mux_quic-t.h>
+#include <haproxy/mux_spop-t.h>
 #include <haproxy/peers-t.h>
 #include <haproxy/quic_conn-t.h>
 #include <haproxy/stconn-t.h>
@@ -41,11 +44,16 @@
 #define SHOW_AS_PEERS 0x00080000
 #define SHOW_AS_PEER  0x00100000
 #define SHOW_AS_QC    0x00200000
+#define SHOW_AS_SPOPC 0x00400000
+#define SHOW_AS_SPOPS 0x00800000
+#define SHOW_AS_QCC   0x01000000
+#define SHOW_AS_QCS   0x02000000
+#define SHOW_AS_APPCTX 0x04000000
 
 // command line names, must be in exact same order as the SHOW_AS_* flags above
 // so that show_as_words[i] matches flag 1U<<i.
 const char *show_as_words[] = { "ana", "chn", "conn", "sc", "stet", "strm", "task", "txn", "sd", "hsl", "htx", "hmsg", "fd", "h2c", "h2s",  "h1c", "h1s", "fconn", "fstrm",
-				"peers", "peer", "qc"};
+				"peers", "peer", "qc", "spopc", "spops", "qcc", "qcs", "appctx"};
 
 /* will be sufficient for even largest flag names */
 static char buf[4096];
@@ -161,6 +169,11 @@ int main(int argc, char **argv)
 		if (show_as & SHOW_AS_PEERS) printf("peers->flags = %s\n",(peers_show_flags  (buf, bsz, " | ", flags), buf));
 		if (show_as & SHOW_AS_PEER)  printf("peer->flags = %s\n", (peer_show_flags   (buf, bsz, " | ", flags), buf));
 		if (show_as & SHOW_AS_QC)    printf("qc->flags = %s\n",   (qc_show_flags     (buf, bsz, " | ", flags), buf));
+		if (show_as & SHOW_AS_SPOPC) printf("spopc->flags = %s\n",(spop_conn_show_flags(buf, bsz, " | ", flags), buf));
+		if (show_as & SHOW_AS_SPOPS) printf("spops->flags = %s\n",(spop_strm_show_flags(buf, bsz, " | ", flags), buf));
+		if (show_as & SHOW_AS_QCC)    printf("qcc->flags = %s\n", (qcc_show_flags    (buf, bsz, " | ", flags), buf));
+		if (show_as & SHOW_AS_QCS)    printf("qcs->flags = %s\n", (qcs_show_flags    (buf, bsz, " | ", flags), buf));
+		if (show_as & SHOW_AS_APPCTX) printf("appctx->flags = %s\n", (appctx_show_flags(buf, bsz, " | ", flags), buf));
 	}
 	return 0;
 }
