@@ -12,6 +12,7 @@ struct acme_cfg {
 	char *filename;             /* config filename */
 	int linenum;                /* config linenum */
 	char *name;                 /* section name */
+	int reuse_key;              /* do we need to renew the private key */
 	char *directory;            /* directory URL */
 	char *map;                  /* storage for tokens + thumbprint */
 	struct {
@@ -27,6 +28,8 @@ struct acme_cfg {
 		int curves;                 /* NID of curves */
 	} key;
 	char *challenge;            /* HTTP-01, DNS-01, etc */
+	char *vars;                 /* variables put in the dpapi sink */
+	char *provider;             /* DNS provider put in the dpapi sink */
 	struct acme_cfg *next;
 };
 
