@@ -16,6 +16,7 @@ enum {
 	QUIC_ST_SOCKET_FULL,
 	QUIC_ST_SENDTO_ERR,
 	QUIC_ST_SENDTO_ERR_UNKNWN,
+	QUIC_ST_SENT_PACKET,
 	QUIC_ST_LOST_PACKET,
 	QUIC_ST_TOO_SHORT_INITIAL_DGRAM,
 	QUIC_ST_RETRY_SENT,
@@ -24,6 +25,8 @@ enum {
 	QUIC_ST_HALF_OPEN_CONN,
 	QUIC_ST_HDSHK_FAIL,
 	QUIC_ST_STATELESS_RESET_SENT,
+	/* Special events of interest */
+	QUIC_ST_CONN_MIGRATION_DONE,
 	/* Transport errors */
 	QUIC_ST_TRANSP_ERR_NO_ERROR,
 	QUIC_ST_TRANSP_ERR_INTERNAL_ERROR,
@@ -47,8 +50,8 @@ enum {
 	/* Stream related counters */
 	QUIC_ST_DATA_BLOCKED,
 	QUIC_ST_STREAM_DATA_BLOCKED,
-	QUIC_ST_STREAMS_DATA_BLOCKED_BIDI,
-	QUIC_ST_STREAMS_DATA_BLOCKED_UNI,
+	QUIC_ST_STREAMS_BLOCKED_BIDI,
+	QUIC_ST_STREAMS_BLOCKED_UNI,
 	QUIC_STATS_COUNT /* must be the last */
 };
 
@@ -60,14 +63,17 @@ struct quic_counters {
 	long long socket_full;       /* total number of EAGAIN errors on sendto() calls */
 	long long sendto_err;        /* total number of errors on sendto() calls, EAGAIN excepted */
 	long long sendto_err_unknown; /* total number of errors on sendto() calls which are currently not supported */
+	long long sent_pkt;          /* total number of sent packets */
 	long long lost_pkt;          /* total number of lost packets */
 	long long too_short_initial_dgram; /* total number of too short datagrams with Initial packets */
 	long long retry_sent;        /* total number of Retry sent */
 	long long retry_validated;   /* total number of validated Retry tokens */
 	long long retry_error;       /* total number of Retry token errors */
-	long long half_open_conn;    /* total number of half open connections */
+	long long half_open_conn;    /* current number of connections waiting for address validation */
 	long long hdshk_fail;        /* total number of handshake failures */
 	long long stateless_reset_sent; /* total number of handshake failures */
+	/* Special events of interest */
+	long long conn_migration_done; /* total number of connection migration handled */
 	/* Transport errors */
 	long long quic_transp_err_no_error; /* total number of NO_ERROR connection errors */
 	long long quic_transp_err_internal_error; /* total number of INTERNAL_ERROR connection errors */
@@ -90,9 +96,9 @@ struct quic_counters {
 	long long quic_transp_err_unknown_error;      /* total number of UNKNOWN_ERROR connection errors */
 	/* Streams related counters */
 	long long data_blocked;              /* total number of times DATA_BLOCKED frame was received */
-	long long stream_data_blocked;       /* total number of times STEAM_DATA_BLOCKED frame was received */
-	long long streams_data_blocked_bidi; /* total number of times STREAMS_DATA_BLOCKED_BIDI frame was received */
-	long long streams_data_blocked_uni;  /* total number of times STREAMS_DATA_BLOCKED_UNI frame was received */
+	long long stream_data_blocked;       /* total number of times STREAM_DATA_BLOCKED frame was received */
+	long long streams_blocked_bidi;      /* total number of times STREAMS_BLOCKED_BIDI frame was received */
+	long long streams_blocked_uni;       /* total number of times STREAMS_BLOCKED_UNI frame was received */
 };
 
 #endif /* USE_QUIC */
