@@ -27,14 +27,14 @@ extern struct signal_descriptor signal_state[];
 __decl_thread(extern HA_SPINLOCK_T signals_lock);
 
 void signal_handler(int sig);
-void __signal_process_queue();
-void deinit_signals();
+void __signal_process_queue(void);
+void deinit_signals(void);
 struct sig_handler *signal_register_fct(int sig, void (*fct)(struct sig_handler *), int arg);
 struct sig_handler *signal_register_task(int sig, struct task *task, int reason);
 void signal_unregister_handler(struct sig_handler *handler);
 void signal_unregister_target(int sig, void *target);
 void signal_unregister(int sig);
-void haproxy_unblock_signals();
+void haproxy_unblock_signals(void);
 
 static inline void signal_process_queue()
 {
