@@ -175,7 +175,7 @@ int mworker_env_to_proc_list()
 
 	while ((token = strtok_r(msg, "|", &s1))) {
 		char *subtoken = NULL;
-		char *s2;
+		char *s2 = NULL;
 
 		msg = NULL;
 
@@ -281,35 +281,8 @@ void mworker_block_signals()
 	ha_sigmask(SIG_SETMASK, &set, NULL);
 }
 
-void mworker_unblock_sigchld()
-{
-	sigset_t set;
-
-	signal_register_fct(SIGCHLD, mworker_catch_sigchld, SIGCHLD);
-
-	sigemptyset(&set);
-	sigaddset(&set, SIGCHLD);
-
-	ha_sigmask(SIG_UNBLOCK, &set, NULL);
-}
-
 void mworker_unblock_signals()
 {
-	signal_unregister(SIGTTIN);
-	signal_unregister(SIGTTOU);
-	signal_unregister(SIGUSR1);
-	signal_unregister(SIGHUP);
-	signal_unregister(SIGQUIT);
-
-	signal_register_fct(SIGTERM, mworker_catch_sigterm, SIGTERM);
-	signal_register_fct(SIGUSR1, mworker_catch_sigterm, SIGUSR1);
-	signal_register_fct(SIGTTIN, mworker_broadcast_signal, SIGTTIN);
-	signal_register_fct(SIGTTOU, mworker_broadcast_signal, SIGTTOU);
-	signal_register_fct(SIGINT, mworker_catch_sigterm, SIGINT);
-	signal_register_fct(SIGHUP, mworker_catch_sighup, SIGHUP);
-	signal_register_fct(SIGUSR2, mworker_catch_sighup, SIGUSR2);
-	signal_register_fct(SIGCHLD, mworker_catch_sigchld, SIGCHLD);
-
 	haproxy_unblock_signals();
 }
 
@@ -512,14 +485,6 @@ void mworker_catch_sigterm(struct sig_handler *sh)
 	mworker_kill(sig);
 }
 
-/* handle operations that can't be done in the signal handler */
-static struct task *mworker_task_child_failure(struct task *task, void *context, unsigned int state)
-{
-	mworker_unblock_signals();
-	task_destroy(task);
-	return NULL;
-}
-
 /*
  * Performs some routines for the worker process, which has failed the reload,
  * updates the global load_status.
@@ -527,7 +492,6 @@ static struct task *mworker_task_child_failure(struct task *task, void *context,
 static void mworker_on_new_child_failure(int exitpid, int status)
 {
 	struct mworker_proc *child;
-	struct task *t;
 
 	/* increment the number of failed reloads */
 	list_for_each_entry(child, &proc_list, list) {
@@ -544,15 +508,6 @@ static void mworker_on_new_child_failure(int exitpid, int status)
 	 * the READY=1 signal still need to be sent */
 	if (global.tune.options & GTUNE_USE_SYSTEMD)
 		sd_notify(0, "READY=1\nSTATUS=Reload failed!\n");
-
-	/* call a task to unblock the signals from outside the sig handler */
-	if ((t = task_new_here()) == NULL) {
-		ha_warning("Can't restore HAProxy signals!\n");
-		return;
-	}
-
-	t->process = mworker_task_child_failure;
-	task_wakeup(t, TASK_WOKEN_MSG);
 }
 
 /*
@@ -827,7 +782,6 @@ void mworker_cleanup_proc()
 
 struct cli_showproc_ctx {
 	int debug;
-	int next_reload; /* reload number to resume from, 0 = from the beginning  */
 };
 
 /*  Displays workers and processes  */
@@ -846,32 +800,23 @@ static int cli_io_handler_show_proc(struct appctx *appctx)
 
 	chunk_reset(&trash);
 
-	if (ctx->next_reload == 0) {
-		memprintf(&reloadtxt, "%d [failed: %d]", proc_self->reloads, proc_self->failedreloads);
-		chunk_printf(&trash, "#%-14s %-15s %-15s %-15s %-15s", "<PID>", "<type>", "<reloads>", "<uptime>", "<version>");
-		if (ctx->debug)
-			chunk_appendf(&trash, "\t\t %-15s %-15s", "<ipc_fd[0]>", "<ipc_fd[1]>");
-		chunk_appendf(&trash, "\n");
-
-		/* display the master only the first time */
-		memprintf(&uptime, "%dd%02dh%02dm%02ds", up / 86400, (up % 86400) / 3600, (up % 3600) / 60, (up % 60));
-		chunk_appendf(&trash, "%-15u %-15s %-15s %-15s %-15s", (unsigned int)getpid(), "master", reloadtxt, uptime, haproxy_version);
-		if (ctx->debug)
-			chunk_appendf(&trash, "\t\t %-15d %-15d", proc_self->ipc_fd[0], proc_self->ipc_fd[1]);
-		chunk_appendf(&trash, "\n");
-	}
+	memprintf(&reloadtxt, "%d [failed: %d]", proc_self->reloads, proc_self->failedreloads);
+	chunk_printf(&trash, "#%-14s %-15s %-15s %-15s %-15s", "<PID>", "<type>", "<reloads>", "<uptime>", "<version>");
+	if (ctx->debug)
+		chunk_appendf(&trash, "\t\t %-15s %-15s", "<ipc_fd[0]>", "<ipc_fd[1]>");
+	chunk_appendf(&trash, "\n");
+	memprintf(&uptime, "%dd%02dh%02dm%02ds", up / 86400, (up % 86400) / 3600, (up % 3600) / 60, (up % 60));
+	chunk_appendf(&trash, "%-15u %-15s %-15s %-15s %-15s", (unsigned int)getpid(), "master", reloadtxt, uptime, haproxy_version);
+	if (ctx->debug)
+		chunk_appendf(&trash, "\t\t %-15d %-15d", proc_self->ipc_fd[0], proc_self->ipc_fd[1]);
+	chunk_appendf(&trash, "\n");
 	ha_free(&reloadtxt);
 	ha_free(&uptime);
 
 	/* displays current processes */
-	if (ctx->next_reload == 0)
-		chunk_appendf(&trash, "# workers\n");
+
+	chunk_appendf(&trash, "# workers\n");
 	list_for_each_entry(child, &proc_list, list) {
-
-		/* don't display current worker if we only need the next ones */
-		if (ctx->next_reload != 0)
-			continue;
-
 		up = date.tv_sec - child->timestamp;
 		if (up < 0) /* must never be negative because of clock drift */
 			up = 0;
@@ -891,22 +836,17 @@ static int cli_io_handler_show_proc(struct appctx *appctx)
 		ha_free(&uptime);
 	}
 
-	if (applet_putchk(appctx, &trash) == -1)
-		return 0;
-
 	/* displays old processes */
-	if (old || ctx->next_reload) { /* there's more */
-		if (ctx->next_reload == 0)
-			chunk_appendf(&trash, "# old workers\n");
+
+	if (old) {
+		char *msg = NULL;
+
+		chunk_appendf(&trash, "# old workers\n");
 		list_for_each_entry(child, &proc_list, list) {
 			up = date.tv_sec - child->timestamp;
 			if (up <= 0) /* must never be negative because of clock drift */
 				up = 0;
 
-			/* If we're resuming, skip entries that were already printed (reload >= ctx->next_reload) */
-			if (ctx->next_reload && child->reloads >= ctx->next_reload)
-				continue;
-
 			if (!(child->options & PROC_O_TYPE_WORKER))
 				continue;
 
@@ -917,17 +857,9 @@ static int cli_io_handler_show_proc(struct appctx *appctx)
 					chunk_appendf(&trash, "\t\t %-15d %-15d", child->ipc_fd[0], child->ipc_fd[1]);
 				chunk_appendf(&trash, "\n");
 				ha_free(&uptime);
-
-				/* Try to flush so we can resume after this reload on next page if the buffer is full. */
-				if (applet_putchk(appctx, &trash) == -1) {
-					/* resume at this reload (exclude it on next pass) */
-					ctx->next_reload = child->reloads; /* resume after entries >= this reload */
-					return 0;
-				}
-				chunk_reset(&trash);
 			}
-
 		}
+		free(msg);
 	}
 
 	/* displays external process */
@@ -970,11 +902,12 @@ static int cli_io_handler_show_proc(struct appctx *appctx)
 		}
 	}
 
+
+
 	if (applet_putchk(appctx, &trash) == -1)
 		return 0;
 
-	/* dump complete: reset resume cursor so next 'show proc' starts from the top */
-	ctx->next_reload = 0;
+	/* dump complete */
 	return 1;
 }
 /* reload the master process */
@@ -1075,7 +1008,6 @@ static int cli_io_handler_show_loadstatus(struct appctx *appctx)
 	else
 		chunk_printf(&trash, "Success=1\n");
 
-#ifdef USE_SHM_OPEN
 	if (startup_logs && ring_data(startup_logs) > 1)
 		chunk_appendf(&trash, "--\n");
 
@@ -1083,14 +1015,10 @@ static int cli_io_handler_show_loadstatus(struct appctx *appctx)
 		return 0;
 
 	if (startup_logs) {
-		appctx->io_handler = NULL;
+		appctx->cli_ctx.io_handler = NULL;
 		ring_attach_cli(startup_logs, appctx, 0);
 		return 0;
 	}
-#else
-	if (applet_putchk(appctx, &trash) == -1)
-		return 0;
-#endif
 	return 1;
 }
 
@@ -1233,7 +1161,23 @@ static void mworker_loop()
 	/* Busy polling makes no sense in the master :-) */
 	global.tune.options &= ~GTUNE_BUSY_POLLING;
 
-	mworker_unblock_sigchld();
+
+	signal_unregister(SIGTTIN);
+	signal_unregister(SIGTTOU);
+	signal_unregister(SIGUSR1);
+	signal_unregister(SIGHUP);
+	signal_unregister(SIGQUIT);
+
+	signal_register_fct(SIGTERM, mworker_catch_sigterm, SIGTERM);
+	signal_register_fct(SIGUSR1, mworker_catch_sigterm, SIGUSR1);
+	signal_register_fct(SIGTTIN, mworker_broadcast_signal, SIGTTIN);
+	signal_register_fct(SIGTTOU, mworker_broadcast_signal, SIGTTOU);
+	signal_register_fct(SIGINT, mworker_catch_sigterm, SIGINT);
+	signal_register_fct(SIGHUP, mworker_catch_sighup, SIGHUP);
+	signal_register_fct(SIGUSR2, mworker_catch_sighup, SIGUSR2);
+	signal_register_fct(SIGCHLD, mworker_catch_sigchld, SIGCHLD);
+
+	mworker_unblock_signals();
 	mworker_cleantasks();
 
 	mworker_catch_sigchld(NULL); /* ensure we clean the children in case
