@@ -1,5 +1,5 @@
 #include <haproxy/errors.h>
-#include <haproxy/ncbuf.h>
+#include <haproxy/ncbmbuf.h>
 #include <haproxy/proxy.h>
 #include <haproxy/quic_conn.h>
 #include <haproxy/quic_sock.h>
@@ -8,6 +8,7 @@
 #include <haproxy/quic_tp.h>
 #include <haproxy/quic_trace.h>
 #include <haproxy/ssl_sock.h>
+#include <haproxy/stats.h>
 #include <haproxy/trace.h>
 
 DECLARE_POOL(pool_head_quic_ssl_sock_ctx, "quic_ssl_sock_ctx", sizeof(struct ssl_sock_ctx));
@@ -424,7 +425,7 @@ static int ha_quic_ossl_crypto_recv_rcd(SSL *ssl,
 {
 	struct quic_conn *qc = SSL_get_ex_data(ssl, ssl_qc_app_data_index);
 	struct quic_enc_level *qel;
-	struct ncbuf *ncbuf = NULL;
+	struct ncbmbuf *ncbuf = NULL;
 	struct quic_cstream *cstream = NULL;
 	ncb_sz_t data = 0;
 
@@ -436,10 +437,10 @@ static int ha_quic_ossl_crypto_recv_rcd(SSL *ssl,
 			continue;
 
 		ncbuf = &cstream->rx.ncbuf;
-		if (ncb_is_null(ncbuf))
+		if (ncbmb_is_null(ncbuf))
 			continue;
 
-		data = ncb_data(ncbuf, 0);
+		data = ncbmb_data(ncbuf, 0);
 		if (data)
 			break;
 	}
@@ -447,9 +448,9 @@ static int ha_quic_ossl_crypto_recv_rcd(SSL *ssl,
 	if (data) {
 		const unsigned char *cdata;
 
-		BUG_ON(ncb_is_null(ncbuf) || !cstream);
+		BUG_ON(ncbmb_is_null(ncbuf) || !cstream);
 		/* <ncbuf> must not be released at this time. */
-		cdata = (const unsigned char *)ncb_head(ncbuf);
+		cdata = (const unsigned char *)ncbmb_head(ncbuf);
 		cstream->rx.offset += data;
 		TRACE_DEVEL("buffered crypto data were provided to TLS stack",
 					QUIC_EV_CONN_PHPKTS, qc, qel);
@@ -481,24 +482,24 @@ static int ha_quic_ossl_crypto_release_rcd(SSL *ssl,
 
 	list_for_each_entry(qel, &qc->qel_list, list) {
 		struct quic_cstream *cstream = qel->cstream;
-		struct ncbuf *ncbuf;
+		struct ncbmbuf *ncbuf;
 		ncb_sz_t data;
 
 		if (!cstream)
 			continue;
 
 		ncbuf = &cstream->rx.ncbuf;
-		if (ncb_is_null(ncbuf))
+		if (ncbmb_is_null(ncbuf))
 			continue;
 
-		data = ncb_data(ncbuf, 0);
+		data = ncbmb_data(ncbuf, 0);
 		if (!data)
 			continue;
 
 		data = data > bytes_read ? bytes_read : data;
-		ncb_advance(ncbuf, data);
+		ncbmb_advance(ncbuf, data);
 		bytes_read -= data;
-		if (ncb_is_empty(ncbuf)) {
+		if (ncbmb_is_empty(ncbuf)) {
 			TRACE_DEVEL("freeing crypto buf", QUIC_EV_CONN_PHPKTS, qc, qel);
 			quic_free_ncbuf(ncbuf);
 		}
@@ -807,12 +808,28 @@ static forceinline void qc_ssl_dump_errors(struct connection *conn)
 int qc_ssl_do_hanshake(struct quic_conn *qc, struct ssl_sock_ctx *ctx)
 {
 	int ret, ssl_err, state;
+	struct ssl_counters *counters = NULL;
+	struct ssl_counters *counters_px = NULL;
 
 	TRACE_ENTER(QUIC_EV_CONN_SSLDATA, qc);
 
 	ret = 0;
 	ssl_err = SSL_ERROR_NONE;
 	state = qc->state;
+
+	if (qc_is_listener(qc)) {
+		counters = EXTRA_COUNTERS_GET(qc->li->extra_counters, &ssl_stats_module);
+		counters_px = EXTRA_COUNTERS_GET(qc->li->bind_conf->frontend->extra_counters_fe,
+		                                 &ssl_stats_module);
+	}
+	else if (ctx->conn) {
+		struct server *srv = __objt_server(ctx->conn->target);
+
+		counters = EXTRA_COUNTERS_GET(srv->extra_counters, &ssl_stats_module);
+		counters_px = EXTRA_COUNTERS_GET(srv->proxy->extra_counters_be,
+		                                 &ssl_stats_module);
+    }
+
 	if (state < QUIC_HS_ST_COMPLETE) {
 		ssl_err = SSL_do_handshake(ctx->ssl);
 		TRACE_PROTO("SSL_do_handshake() called", QUIC_EV_CONN_IO_CB, qc, NULL, NULL, ctx->ssl);
@@ -882,6 +899,7 @@ int qc_ssl_do_hanshake(struct quic_conn *qc, struct ssl_sock_ctx *ctx)
 
 #ifndef HAVE_OPENSSL_QUIC
 		TRACE_PROTO("SSL handshake OK", QUIC_EV_CONN_IO_CB, qc, &state);
+		ssl_sock_update_counters(ctx->ssl, counters, counters_px, !qc_is_listener(qc));
 #else
 		/* Hack to support O-RTT with the OpenSSL 3.5 QUIC API.
 		 * SSL_do_handshake() succeeds at the first call. Why? |-(
@@ -899,6 +917,7 @@ int qc_ssl_do_hanshake(struct quic_conn *qc, struct ssl_sock_ctx *ctx)
 		}
 		else {
 			TRACE_PROTO("SSL handshake OK", QUIC_EV_CONN_IO_CB, qc, &state);
+			ssl_sock_update_counters(ctx->ssl, counters, counters_px, !qc_is_listener(qc));
 		}
 #endif
 
@@ -975,7 +994,7 @@ int qc_ssl_do_hanshake(struct quic_conn *qc, struct ssl_sock_ctx *ctx)
  * Remaining parameter are there for debugging purposes.
  * Return 1 if succeeded, 0 if not.
  */
-static int qc_ssl_provide_quic_data(struct ncbuf *ncbuf,
+static int qc_ssl_provide_quic_data(struct ncbmbuf *ncbuf,
                                     enum ssl_encryption_level_t level,
                                     struct ssl_sock_ctx *ctx,
                                     const unsigned char *data, size_t len)
@@ -1005,17 +1024,12 @@ static int qc_ssl_provide_quic_data(struct ncbuf *ncbuf,
 	/* The CRYPTO data are consumed even in case of an error to release
 	 * the memory asap.
 	 */
-	if (!ncb_is_null(ncbuf)) {
+	if (!ncbmb_is_null(ncbuf)) {
 #ifdef DEBUG_STRICT
-		ncb_ret = ncb_advance(ncbuf, len);
-		/* ncb_advance() must always succeed. This is guaranteed as
-		 * this is only done inside a data block. If false, this will
-		 * lead to handshake failure with quic_enc_level offset shifted
-		 * from buffer data.
-		 */
+		ncb_ret = ncbmb_advance(ncbuf, len);
 		BUG_ON(ncb_ret != NCB_RET_OK);
 #else
-		ncb_advance(ncbuf, len);
+		ncbmb_advance(ncbuf, len);
 #endif
 	}
 
@@ -1030,7 +1044,7 @@ int qc_ssl_provide_all_quic_data(struct quic_conn *qc, struct ssl_sock_ctx *ctx)
 {
 	int ret = 0;
 	struct quic_enc_level *qel;
-	struct ncbuf *ncbuf;
+	struct ncbmbuf *ncbuf;
 	ncb_sz_t data;
 
 	TRACE_ENTER(QUIC_EV_CONN_PHPKTS, qc);
@@ -1041,12 +1055,12 @@ int qc_ssl_provide_all_quic_data(struct quic_conn *qc, struct ssl_sock_ctx *ctx)
 			continue;
 
 		ncbuf = &cstream->rx.ncbuf;
-		if (ncb_is_null(ncbuf))
+		if (ncbmb_is_null(ncbuf))
 			continue;
 
 		/* TODO not working if buffer is wrapping */
-		while ((data = ncb_data(ncbuf, 0))) {
-			const unsigned char *cdata = (const unsigned char *)ncb_head(ncbuf);
+		while ((data = ncbmb_data(ncbuf, 0))) {
+			const unsigned char *cdata = (const unsigned char *)ncbmb_head(ncbuf);
 
 			if (!qc_ssl_provide_quic_data(&qel->cstream->rx.ncbuf, qel->level,
 			                              ctx, cdata, data))
@@ -1057,7 +1071,7 @@ int qc_ssl_provide_all_quic_data(struct quic_conn *qc, struct ssl_sock_ctx *ctx)
 			            QUIC_EV_CONN_PHPKTS, qc, qel);
 		}
 
-		if (!ncb_is_null(ncbuf) && ncb_is_empty(ncbuf)) {
+		if (!ncbmb_is_null(ncbuf) && ncbmb_is_empty(ncbuf)) {
 			TRACE_DEVEL("freeing crypto buf", QUIC_EV_CONN_PHPKTS, qc, qel);
 			quic_free_ncbuf(ncbuf);
 		}
